@@ -1647,10 +1647,10 @@ def _get_list_values_for_import(db_map, data, make_cache, unparse_value):
         max_index = max_indexes.get(list_id)
         if max_index is not None:
             index = max_index + 1
-        elif value_index_list is None:
+        elif not value_index_list:
             index = 0
         else:
-            index = int(max(value_index_list.split(","))) + 1
+            index = max(map(int, value_index_list.split(","))) + 1
         item = {"parameter_value_list_id": list_id, "value": val, "type": type_, "index": index}
         try:
             check_list_value(item, list_names_by_id, list_value_ids_by_index, list_value_ids_by_value)
