@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -18,12 +18,10 @@ Contains the SpineDBClient class.
 
 from urllib.parse import urlparse
 import socket
-import json
 from sqlalchemy.engine.url import URL
-from .helpers import ReceiveAllMixing, encode, decode
+from .server_client_helpers import ReceiveAllMixing, encode, decode
 
-
-client_version = 1
+client_version = 6
 
 
 class SpineDBClient(ReceiveAllMixing):
@@ -39,7 +37,7 @@ class SpineDBClient(ReceiveAllMixing):
     def from_server_url(cls, url):
         parsed = urlparse(url)
         if parsed.scheme != "http":
-            return url
+            raise ValueError(f"unable to create client, invalid server url {url}")
         return cls((parsed.hostname, parsed.port))
 
     def get_db_url(self):
@@ -49,11 +47,29 @@ class SpineDBClient(ReceiveAllMixing):
         """
         return self._send("get_db_url")
 
-    def open_connection(self):
-        return self._send("open_connection")
+    def db_checkin(self):
+        return self._send("db_checkin")
 
-    def close_connection(self):
-        return self._send("close_connection")
+    def db_checkout(self):
+        return self._send("db_checkout")
+
+    def cancel_db_checkout(self):
+        return self._send("cancel_db_checkout")
+
+    def import_data(self, data, comment):
+        return self._send("import_data", args=(data, comment))
+
+    def export_data(self, **kwargs):
+        return self._send("export_data", kwargs=kwargs)
+
+    def call_method(self, method_name, *args, **kwargs):
+        return self._send("call_method", args=(method_name, *args), kwargs=kwargs)
+
+    def open_db_map(self, db_url, upgrade, memory):
+        return self._send("open_db_map", args=(db_url, upgrade, memory))
+
+    def close_db_map(self):
+        return self._send("close_db_map")
 
     def _send(self, request, args=None, kwargs=None, receive=True):
         """
