@@ -16,13 +16,11 @@ Contains the SpineDBServer class.
 :date:   2.2.2020
 """
 
-import os
-from tempfile import gettempdir
 from urllib.parse import urlunsplit
 from contextlib import contextmanager
 import socketserver
 import multiprocessing as mp
-from multiprocessing.connection import Listener, Client
+from multiprocessing.queues import Queue as MPQueue
 import threading
 import atexit
 import traceback
@@ -43,7 +41,6 @@ from .filters.tools import apply_filter_stack
 from .spine_db_client import SpineDBClient
 
 _required_client_version = 6
-_listening = mp.current_process().name == "MainProcess"
 
 
 def _parse_value(v, value_type=None):
@@ -60,85 +57,102 @@ def _unparse_value(value_and_type):
     return dump_db_value(value_and_type)
 
 
-class _Executor:
-    _CLOSE = "close"
-
-    def __init__(self):
-        self._process = None
-        self._address = None
-
-    def _setup(self):
-        self._address = self._make_address()
-        if not _listening:
-            return
-        self._process = mp.Process(target=self._do_work)
-        self._process.start()
-        atexit.register(self.tear_down)
-
-    def tear_down(self):
-        with Client(self._address) as conn:
-            conn.send(self._CLOSE)
-        self._process.join()
-
-    def _make_address(self):
-        cls_name = type(self).__name__
-        pid = os.getpid() if _listening else os.getppid()
-        if os.name == "nt":
-            return rf"\\.\pipe\{cls_name}{pid}"
-        address = os.path.join(gettempdir(), f"{cls_name}{pid}.s")
-        if _listening and os.path.exists(address):
-            os.remove(address)
-        return address
-
-    @property
-    def _handlers(self):
-        raise NotImplementedError
-
-    def _run_request(self, request, *args, **kwargs):
-        with Client(self._address) as conn:
-            conn.send((request, args, kwargs))
-            return conn.recv()
-
-    def _do_work(self):
-        with Listener(self._address) as listener:
-            while True:
-                with listener.accept() as conn:
-                    msg = conn.recv()
-                    if msg == self._CLOSE:
-                        break
-                    request, args, kwargs = msg
-                    handler = self._handlers[request]
-                    result = handler(*args, **kwargs)
-                    conn.send(result)
+class SpineDBServer(socketserver.TCPServer):
+    def __init__(self, manager_queue, ordering, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.manager_queue = manager_queue
+        self.ordering = ordering
 
 
-class _OrderingManager(_Executor):
-    _CHECKOUT_COMPLETE = object()
+class _DeepCopyableQueue(MPQueue):
+    def __init__(self, *args, **kwargs):
+        ctx = mp.get_context()
+        super().__init__(*args, **kwargs, ctx=ctx)
+
+    def __deepcopy__(self, _memo):
+        return self
+
+
+class _DBServerManager:
+    _SHUTDOWN = "shutdown"
+    _CHECKOUT_COMPLETE = "checkout_complete"
 
     def __init__(self):
         super().__init__()
-        self._orderings = {}
+        self._servers = {}
         self._checkouts = {}
         self._waiters = {}
-        self._setup()
+        self._queue = _DeepCopyableQueue()
+        self._process = mp.Process(target=self._do_work)
+        self._process.start()
+
+    def shutdown(self):
+        self._queue.put(self._SHUTDOWN)
+        self._process.join()
+
+    @property
+    def queue(self):
+        return self._queue
 
     @property
     def _handlers(self):
         return {
-            "register_ordering": self._do_register_ordering,
-            "db_checkin": self._do_db_checkin,
-            "db_checkout": self._do_db_checkout,
-            "quick_db_checkout": self._do_quick_db_checkout,
-            "cancel_db_checkout": self._do_cancel_db_checkout,
+            "start_server": self._start_server,
+            "shutdown_server": self._shutdown_server,
+            "shutdown_servers": self._shutdown_servers,
+            "db_checkin": self._db_checkin,
+            "db_checkout": self._db_checkout,
+            "quick_db_checkout": self._quick_db_checkout,
+            "cancel_db_checkout": self._cancel_db_checkout,
         }
 
-    def _do_register_ordering(self, server_address, ordering):
-        self._orderings[server_address] = ordering
+    def _do_work(self):
+        while True:
+            msg = self._queue.get()
+            if msg == self._SHUTDOWN:
+                break
+            output_queue, request, args, kwargs = msg
+            handler = self._handlers[request]
+            result = handler(*args, **kwargs)
+            output_queue.put(result)
+        for server_address in list(self._servers):
+            self._shutdown_server(server_address)
 
-    def _do_db_checkin(self, server_address):
-        ordering = self._orderings.get(server_address)
-        if not ordering:
+    def _start_server(self, db_url, upgrade, memory, ordering):
+        host = "127.0.0.1"
+        while True:
+            with socketserver.TCPServer((host, 0), None) as s:
+                port = s.server_address[1]
+            try:
+                server = SpineDBServer(self._queue, ordering, (host, port), DBRequestHandler)
+                break
+            except OSError:
+                # [Errno 98] Address already in use
+                time.sleep(0.02)
+        self._servers[server.server_address] = server
+        server_thread = threading.Thread(target=server.serve_forever)
+        server_thread.daemon = True
+        server_thread.start()
+        SpineDBClient(server.server_address).open_db_map(db_url, upgrade, memory)
+        return server.server_address
+
+    def _shutdown_server(self, server_address):
+        server = self._servers.pop(server_address, None)
+        if server is None:
+            return False
+        SpineDBClient(server.server_address).close_db_map()
+        server.shutdown()
+        server.server_close()
+        return True
+
+    def _shutdown_servers(self):
+        return all(self._shutdown_server(server_address) for server_address in list(self._servers))
+
+    def _db_checkin(self, server_address):
+        server = self._servers.get(server_address)
+        if not server:
             return
+        ordering = server.ordering
         checkouts = self._checkouts.get(ordering["id"], dict())
         full_checkouts = set(x for x, count in checkouts.items() if count == self._CHECKOUT_COMPLETE)
         precursors = ordering["precursors"]
@@ -148,13 +162,14 @@ class _OrderingManager(_Executor):
         self._waiters.setdefault(ordering["id"], {})[event] = precursors
         return event
 
-    def _do_db_checkout(self, server_address):
-        ordering = self._orderings.get(server_address)
-        if not ordering:
+    def _db_checkout(self, server_address):
+        server = self._servers.get(server_address)
+        if not server:
             return
-        self._do_quick_db_checkout(ordering)
+        ordering = server.ordering
+        self._quick_db_checkout(ordering)
 
-    def _do_quick_db_checkout(self, ordering):
+    def _quick_db_checkout(self, ordering):
         current = ordering["current"]
         checkouts = self._checkouts.setdefault(ordering["id"], dict())
         if current not in checkouts:
@@ -170,13 +185,33 @@ class _OrderingManager(_Executor):
             del waiters[event]
             event.set()
 
-    def _do_cancel_db_checkout(self, server_address):
-        ordering = self._orderings.get(server_address)
-        if not ordering:
+    def _cancel_db_checkout(self, server_address):
+        server = self._servers.get(server_address)
+        if not server:
             return
+        ordering = server.ordering
         checkouts = self._checkouts.get(ordering["id"], dict())
         checkouts.pop(ordering["current"], None)
 
+
+class _ManagerRequestHandler:
+    def __init__(self, mngr_queue):
+        self._mngr_queue = mngr_queue
+
+    def _run_request(self, request, *args, **kwargs):
+        output_queue = mp.Manager().Queue()
+        self._mngr_queue.put((output_queue, request, args, kwargs))
+        return output_queue.get()
+
+    def start_server(self, db_url, upgrade, memory, ordering):
+        return self._run_request("start_server", db_url, upgrade, memory, ordering)
+
+    def shutdown_server(self, server_address):
+        return self._run_request("shutdown_server", server_address)
+
+    def shutdown_servers(self):
+        return self._run_request("shutdown_servers")
+
     def register_ordering(self, server_address, ordering):
         return self._run_request("register_ordering", server_address, ordering)
 
@@ -195,13 +230,6 @@ class _OrderingManager(_Executor):
         return self._run_request("cancel_db_checkout", server_address)
 
 
-_ordering_manager = _OrderingManager()
-
-
-def quick_db_checkout(ordering):
-    _ordering_manager.quick_db_checkout(ordering)
-
-
 class _DBWorker:
     _CLOSE = "close"
 
@@ -224,17 +252,14 @@ class _DBWorker:
     def db_url(self):
         return str(self._db_map.db_url)
 
-    def tear_down(self):
+    def shutdown(self):
         self._in_queue.put(self._CLOSE)
         self._thread.join()
 
     def _do_work(self):
         try:
             self._db_map = DatabaseMapping(
-                self._db_url,
-                upgrade=self._upgrade,
-                memory=self._memory,
-                create=self._create,
+                self._db_url, upgrade=self._upgrade, memory=self._memory, create=self._create
             )
             self._out_queue.put(None)
         except Exception as error:  # pylint: disable=broad-except
@@ -317,7 +342,7 @@ class _DBManager:
         worker = self._workers.pop(server_address, None)
         if worker is None:
             return dict(result=False)
-        worker.tear_down()
+        worker.shutdown()
         return dict(result=True)
 
     def get_db_url(self, server_address):
@@ -405,11 +430,9 @@ class HandleDBMixin:
 
     def apply_filters(self, filters):
         configs = [
-            {
-                "scenario": scenario_filter_config,
-                "tool": tool_filter_config,
-                "alternatives": alternative_filter_config,
-            }[key](value)
+            {"scenario": scenario_filter_config, "tool": tool_filter_config, "alternatives": alternative_filter_config}[
+                key
+            ](value)
             for key, value in filters.items()
         ]
         return _db_manager.apply_filters(self.server_address, configs)
@@ -418,15 +441,15 @@ class HandleDBMixin:
         return _db_manager.clear_filters(self.server_address)
 
     def db_checkin(self):
-        _ordering_manager.db_checkin(self.server_address)
+        _ManagerRequestHandler(self.server_manager_queue).db_checkin(self.server_address)
         return dict(result=True)
 
     def db_checkout(self):
-        _ordering_manager.db_checkout(self.server_address)
+        _ManagerRequestHandler(self.server_manager_queue).db_checkout(self.server_address)
         return dict(result=True)
 
     def cancel_db_checkout(self):
-        _ordering_manager.cancel_db_checkout(self.server_address)
+        _ManagerRequestHandler(self.server_manager_queue).cancel_db_checkout(self.server_address)
         return dict(result=True)
 
     def open_db_map(self, db_url, upgrade, memory):
@@ -440,10 +463,7 @@ class HandleDBMixin:
         # NOTE: Clients should always send requests "get_api_version" and "get_db_url" in a format that is compatible
         # with the legacy server -- to (based on the format of the answer) determine that it needs to be updated.
         # That's why we don't expand the extras so far.
-        response = {
-            "get_api_version": spinedb_api_version,
-            "get_db_url": self.get_db_url(),
-        }.get(request)
+        response = {"get_api_version": spinedb_api_version, "get_db_url": self.get_db_url()}.get(request)
         if response is not None:
             return response
         try:
@@ -469,7 +489,7 @@ class HandleDBMixin:
             return dict(error=f"invalid request '{request}'")
         try:
             return handler(*args, **kwargs)
-        except Exception as e:  # pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             return dict(error=traceback.format_exc())
 
     def handle_request(self, request):
@@ -496,74 +516,21 @@ class DBRequestHandler(ReceiveAllMixing, HandleDBMixin, socketserver.BaseRequest
     def server_address(self):
         return self.server.server_address
 
+    @property
+    def server_manager_queue(self):
+        return self.server.manager_queue
+
     def handle(self):
         request = self._recvall()
         response = self.handle_request(request)
         self.request.sendall(response + bytes(self._EOT, self._ENCODING))
 
 
-class _ServerManager(_Executor):
-    def __init__(self):
-        super().__init__()
-        self._servers = {}
-        self._setup()
-
-    @property
-    def _handlers(self):
-        return {
-            "start_server": self._do_start_server,
-            "shutdown_server": self._do_shutdown_server,
-            "shutdown_servers": self._do_shutdown_servers,
-        }
-
-    def _do_start_server(self, db_url, upgrade, memory):
-        host = "127.0.0.1"
-        while True:
-            with socketserver.TCPServer((host, 0), None) as s:
-                port = s.server_address[1]
-            try:
-                server = socketserver.TCPServer((host, port), DBRequestHandler)
-                break
-            except OSError:
-                # [Errno 98] Address already in use
-                time.sleep(0.02)
-        self._servers[server.server_address] = server
-        server_thread = threading.Thread(target=server.serve_forever)
-        server_thread.daemon = True
-        server_thread.start()
-        SpineDBClient(server.server_address).open_db_map(db_url, upgrade, memory)
-        return server.server_address
-
-    def _do_shutdown_server(self, server_address):
-        server = self._servers.pop(server_address, None)
-        if server is None:
-            return False
-        SpineDBClient(server.server_address).close_db_map()
-        server.shutdown()
-        server.server_close()
-        return True
-
-    def _do_shutdown_servers(self):
-        return all(self._do_shutdown_server(server_address) for server_address in list(self._servers))
-
-    def start_server(self, db_url, upgrade, memory):
-        return self._run_request("start_server", db_url, upgrade, memory)
-
-    def shutdown_server(self, server_address):
-        return self._run_request("shutdown_server", server_address)
-
-    def shutdown_servers(self):
-        return self._run_request("shutdown_servers")
-
-    def tear_down(self):
-        self.shutdown_servers()
-        super().tear_down()
+def quick_db_checkout(server_manager_queue, ordering):
+    _ManagerRequestHandler(server_manager_queue).quick_db_checkout(ordering)
 
 
-_server_manager = _ServerManager()
-
-
-def start_spine_db_server(db_url, upgrade=False, memory=False, ordering=None):
+def start_spine_db_server(server_manager_queue, db_url, upgrade=False, memory=False, ordering=None):
     """
     Args:
         db_url (str): Spine db url
@@ -573,20 +540,30 @@ def start_spine_db_server(db_url, upgrade=False, memory=False, ordering=None):
     Returns:
         tuple: server address (e.g. (127.0.0.1, 54321))
     """
-    server_address = _server_manager.start_server(db_url, upgrade, memory)
-    _ordering_manager.register_ordering(server_address, ordering)
+    handler = _ManagerRequestHandler(server_manager_queue)
+    server_address = handler.start_server(db_url, upgrade, memory, ordering)
     return server_address
 
 
-def shutdown_spine_db_server(server_address):
-    _server_manager.shutdown_server(server_address)
+def shutdown_spine_db_server(server_manager_queue, server_address):
+    handler = _ManagerRequestHandler(server_manager_queue)
+    handler.shutdown_server(server_address)
 
 
 @contextmanager
-def closing_spine_db_server(db_url, upgrade=False, memory=False, ordering=None):
-    server_address = start_spine_db_server(db_url, memory=memory, ordering=ordering)
+def closing_spine_db_server(server_manager_queue, db_url, upgrade=False, memory=False, ordering=None):
+    server_address = start_spine_db_server(server_manager_queue, db_url, memory=memory, ordering=ordering)
     host, port = server_address
     try:
         yield urlunsplit(("http", f"{host}:{port}", "", "", ""))
     finally:
-        shutdown_spine_db_server(server_address)
+        shutdown_spine_db_server(server_manager_queue, server_address)
+
+
+@contextmanager
+def db_server_manager():
+    mngr = _DBServerManager()
+    try:
+        yield mngr.queue
+    finally:
+        mngr.shutdown()
